@@ -3,12 +3,14 @@ use Mojo::Base 'Mojo::EventEmitter';
 
 use IO::Socket::SSL;
 use Mojo::IOLoop;
+use Mojo::JSON ();
 use Mojo::Log;
 use Mojo::UserAgent;
+use Scalar::Util ();
 
 use constant DEBUG => $ENV{MOJO_SLACKRTM_DEBUG};
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 has ioloop => sub { Mojo::IOLoop->singleton };
 has ua => sub { Mojo::UserAgent->new };
@@ -204,9 +206,18 @@ sub call_api {
             $slack->log->warn("$method: $error");
         }
     };
+
+    # Data structures like "attachments" need to be serialized to JSON
+    for my $key (keys %$param) {
+        if (ref $param->{$key} && !Scalar::Util::blessed($param->{$key})) {
+            $param->{$key} = Mojo::JSON::to_json($param->{$key});
+        }
+    }
+
     $param->{token} = $self->token unless exists $param->{token};
 
     DEBUG and $self->log->debug("===> call api '$method'");
+    DEBUG and $self->_dump( $param );
     my $url = "$SLACK_URL/$method";
     $self->ua->post($url => form => $param => sub {
         (undef, my $tx) = @_;
